(local { : view } (require :fennel))

(macro expect [text actual expected]
  `(let [actual# ,actual]
     (or (match actual#
           ,expected true
           _# false)
         (assert false (view {
                         :text ,text
                         :expected ,expected
                         :actual actual#
                         })))))

(fn command [rover commands]
  (match rover.direction
    :n {:x rover.x :y (+ rover.y 1) :direction :n}
    :s {:x rover.x :y (- rover.y 1) :direction :s}
    :w {:x (- rover.x 1) :y rover.y :direction :w}
    :e {:x (+ rover.x 1) :y rover.y :direction :e}))

(fn rover [x y direction]
  {: x
   : y
   : direction
   : command
   })

(let [r (rover 7 15 :n)]
  (expect
   "rover knows (x,y) and the direction (N,S,E,W) it is facing"
   r
   {:x 7 :y 15 :direction :n}))

(let [r (rover 0 0 :n)]
  (expect "The rover receives a character array of commands"
          (r:command [:f :f :f])
          {}))

(let [r (rover 0 0 :n)]
  (expect "Moves north when pointing north and asked to move forward"
          (r:command [:f]) {:x 0 :y 1 :direction :n}))

(let [r (rover 3 2 :s)]
  (expect "Moves south when pointing south and asked to move forward"
          (r:command [:f]) {:x 3 :y 1 :direction :s}))

(let [r (rover 0 0 :w)]
  (expect "Moves west when pointing west and asked to move forward"
          (r:command [:f]) {:x -1 :y 0 :direction :w}))

(let [r (rover 0 0 :e)]
  (expect "Moves east when pointing east and asked to move forward"
          (r:command [:f]) {:x 1 :y 0 :direction :e}))

(print "OK")
